@@ -209,19 +209,19 @@ bool WINAPI SFileOpenArchive(
     {
         ULONGLONG SearchOffset = 0;
         ULONGLONG EndOfSearch = FileSize;
-        DWORD dwStreamFlags = 0;
+        DWORD dwStrmFlags = 0;
         DWORD dwHeaderSize;
         DWORD dwHeaderID;
         bool bSearchComplete = false;
 
         memset(ha, 0, sizeof(TMPQArchive));
-        ha->pfnHashString = HashString;
+        ha->pfnHashString = HashStringSlash;
         ha->pStream = pStream;
         pStream = NULL;
 
         // Set the archive read only if the stream is read-only
-        FileStream_GetFlags(ha->pStream, &dwStreamFlags);
-        ha->dwFlags |= (dwStreamFlags & STREAM_FLAG_READ_ONLY) ? MPQ_FLAG_READ_ONLY : 0;
+        FileStream_GetFlags(ha->pStream, &dwStrmFlags);
+        ha->dwFlags |= (dwStrmFlags & STREAM_FLAG_READ_ONLY) ? MPQ_FLAG_READ_ONLY : 0;
 
         // Also remember if we shall check sector CRCs when reading file
         ha->dwFlags |= (dwFlags & MPQ_OPEN_CHECK_SECTOR_CRC) ? MPQ_FLAG_CHECK_SECTOR_CRC : 0;
@@ -365,6 +365,13 @@ bool WINAPI SFileOpenArchive(
             ha->pUserData = NULL;
         }
 
+        // Anti-overflow. If the hash table size in the header is
+        // higher than 0x10000000, it would overflow in 32-bit version
+        // Observed in the malformed Warcraft III maps
+        // Example map: MPQ_2016_v1_ProtectedMap_TableSizeOverflow.w3x
+        ha->pHeader->dwBlockTableSize = (ha->pHeader->dwBlockTableSize & BLOCK_INDEX_MASK);
+        ha->pHeader->dwHashTableSize = (ha->pHeader->dwHashTableSize & BLOCK_INDEX_MASK);
+
         // Both MPQ_OPEN_NO_LISTFILE or MPQ_OPEN_NO_ATTRIBUTES trigger read only mode
         if(dwFlags & (MPQ_OPEN_NO_LISTFILE | MPQ_OPEN_NO_ATTRIBUTES))
             ha->dwFlags |= MPQ_FLAG_READ_ONLY;
@@ -428,7 +435,7 @@ bool WINAPI SFileOpenArchive(
         if(pFileEntry != NULL)
         {
             // Just remember that the archive is weak-signed
-            assert(pFileEntry->dwFlags == MPQ_FILE_EXISTS);
+            assert((pFileEntry->dwFlags & MPQ_FILE_EXISTS) != 0);
             ha->dwFileFlags3 = pFileEntry->dwFlags;
         }
 
