@@ -71,6 +71,7 @@
 /* 27.08.14  9.10  Lad  Signing archives with weak digital signature         */
 /* 25.11.14  9.11  Lad  Fixed bug reading & creating HET table               */
 /* 18.09.15  9.20  Lad  Release 9.20                                         */
+/* 12.12.16  9.20  Lad  Release 9.21                                         */
 /*****************************************************************************/
 
 #ifndef __STORMLIB_H__
@@ -135,8 +136,8 @@ extern "C" {
 //-----------------------------------------------------------------------------
 // Defines
 
-#define STORMLIB_VERSION                0x0914  // Current version of StormLib (9.20)
-#define STORMLIB_VERSION_STRING         "9.20"  // String version of StormLib version
+#define STORMLIB_VERSION                0x0915  // Current version of StormLib (9.21)
+#define STORMLIB_VERSION_STRING         "9.21"  // String version of StormLib version
 
 #define ID_MPQ                      0x1A51504D  // MPQ archive header ID ('MPQ\x1A')
 #define ID_MPQ_USERDATA             0x1B51504D  // MPQ userdata entry ('MPQ\x1B')
@@ -213,12 +214,11 @@ extern "C" {
 #define MPQ_FILE_DELETE_MARKER      0x02000000  // File is a deletion marker. Used in MPQ patches, indicating that the file no longer exists.
 #define MPQ_FILE_SECTOR_CRC         0x04000000  // File has checksums for each sector.
                                                 // Ignored if file is not compressed or imploded.
-
-#define MPQ_FILE_COMPRESS_MASK      0x0000FF00  // Mask for a file being compressed
+#define MPQ_FILE_SIGNATURE          0x10000000  // Present on STANDARD.SNP\(signature). The only occurence ever observed
 #define MPQ_FILE_EXISTS             0x80000000  // Set if file exists, reset when the file was deleted
 #define MPQ_FILE_REPLACEEXISTING    0x80000000  // Replace when the file exist (SFileAddFile)
 
-#define MPQ_FILE_EXISTS_MASK        0xF00000FF  // These must be either zero or MPQ_FILE_EXISTS
+#define MPQ_FILE_COMPRESS_MASK      0x0000FF00  // Mask for a file being compressed
 
 #define MPQ_FILE_VALID_FLAGS     (MPQ_FILE_IMPLODE       |  \
                                   MPQ_FILE_COMPRESS      |  \
@@ -228,8 +228,16 @@ extern "C" {
                                   MPQ_FILE_SINGLE_UNIT   |  \
                                   MPQ_FILE_DELETE_MARKER |  \
                                   MPQ_FILE_SECTOR_CRC    |  \
+                                  MPQ_FILE_SIGNATURE     |  \
                                   MPQ_FILE_EXISTS)
 
+// We need to mask out the upper 4 bits of the block table index.
+// This is because it gets shifted out when calculating block table offset
+// BlockTableOffset = pHash->dwBlockIndex << 0x04
+// Malformed MPQ maps may contain block indexes like 0x40000001 or 0xF0000023
+#define BLOCK_INDEX_MASK          0x0FFFFFFF
+#define MPQ_BLOCK_INDEX(pHash) (pHash->dwBlockIndex & BLOCK_INDEX_MASK)
+
 // Compression types for multiple compressions
 #define MPQ_COMPRESSION_HUFFMANN          0x01  // Huffmann compression (used on WAVE files only)
 #define MPQ_COMPRESSION_ZLIB              0x02  // ZLIB compression
@@ -434,6 +442,7 @@ typedef enum _SFileInfoClass
     SFileInfoFlags,                         // File flags from (DWORD)
     SFileInfoEncryptionKey,                 // File encryption key
     SFileInfoEncryptionKeyRaw,              // Unfixed value of the file key
+    SFileInfoCRC32,                         // CRC32 of the file
 } SFileInfoClass;
 
 //-----------------------------------------------------------------------------
@@ -609,12 +618,13 @@ typedef struct _TMPQHash
 
     // The platform the file is used for. 0 indicates the default platform.
     // No other values have been observed.
-    // Note: wPlatform is actually just BYTE, but since it has never been used, we don't care.
-    USHORT wPlatform;
+    BYTE   Platform;
+    BYTE   Reserved;
 
 #else
 
-    USHORT wPlatform;
+    BYTE   Platform;
+    BYTE   Reserved;
     USHORT lcLocale;
 
 #endif
@@ -1076,8 +1086,8 @@ int    WINAPI SCompDecompress2(void * pvOutBuffer, int * pcbOutBuffer, void * pv
 
 #ifndef PLATFORM_WINDOWS
 
-void  SetLastError(int err);
-int   GetLastError();
+void  SetLastError(DWORD err);
+DWORD   GetLastError();
 
 #endif
 
